% plotArclengthN   plot a function in S \subset H^{1/2}(\Gamma) defined by 
%                  its coefficients  with respect to the Lobatto shape
%                  functions over the arclenth of \Gamma. 
%
%   plotArclengthN(coordinates,elements,p,x [,uD] [,options] [,fig])
%   draws the solution given by the coefficient vector x for a boundary 
%   \Gamma which is specified by coordinates and elements. p (either a vector 
%   of length size(elements,1) of an integer >=1) species the polynomial 
%   degrees for the ansatz space S. 
%   uD is a function handle for the exact function uD, options is a
%   struct containing the Lame parameters and fig is the number of the
%   figure in which the plot should be created
%
%  Examples: 
%    elements    = [1,2; 2,3; 3,4];
%    coordinates = [0,0; 1,0; 1,1; 0,1];
%    x = [0,1,1,0,0,0,0].';
%    p = [1,2,3];
%    addpath('../laplace/mat') 
%    addpath('./c')
%    plotArclengthN(coordinates,elements,p,x, @(x,options) x(:,1));
%
% (c) epsBEM team 2012

function plotArclengthN(coordinates,elements,p,x,varargin)

%*** check the optional input arguments
if nargin < 4
  error('At least four input arguments required.')
else
  options = [];
  fig = 0;
  uD = false;
  for k = 1:nargin - 4
    if isstruct(varargin{k})
      options = varargin{k};
    elseif isa(varargin{k}, 'function_handle')
      uD = varargin{k};
    elseif isscalar(varargin{k}) && isnumeric(varargin{k})
      fig = varargin{k};
    end
  end
end


if fig > 0
    figure(fig);
elseif fig == 0
    cla(gca);
end

nC = length(unique(elements)); % number of coordinates
nE = size(elements,1);         % number of elements
dim = size(x,2);               % problem dimension
p = p*ones(nE,1)';

%*** set parameters for the legend
legstr = 'leg = legend(';
if dim==1
  legstr = [legstr,'''u_h'''];
  plot(0,0); hold on
else
  legstr = [legstr,'''u_{h,1}'',''u_{h,2}'''];
  plot(0,0,'b', 0,0,'r'); hold on
end
if isa(uD,'function_handle')
  if dim ==1 
    plot(0,0,'--c')
    legstr = [legstr,',''u'''];
  else
    legstr = [legstr,',''u_{1}'',''u_{2}'''];
    plot(0,0,'--c',0,0,'--m')
  end  
end


%*** compute local mesh-size
h = sqrt(sum((coordinates(elements(:,1),:)-...
              coordinates(elements(:,2),:)).^2,2));
space=0;
s=0;
cp = cumsum([size(coordinates,1);p(:)-1]);

%*** evaluate Lobatto shape functions
pts = linspace(-1,1,2*(max(p)+1));
plob = bLobatto(pts,max(p));

indelem = 1:size(elements,1);
while ~isempty(elements)
    u_sx = [];
    s_sx = [];
    %*** evaluation points for u per element w.r.t. reference element [0,1]
    evalpts = linspace(0,1,50);
    %*** find connected piece of the boundary Gamma
    idx=findCycle(elements);
    s = [0;cumsum(h(indelem(idx)))]+s(end)+space;
    space = ceil(0.05*sum(h));
  
    for k = 1:length(idx)
        %*** plot discrete solution over arclength
        index=[elements(idx(k),:),cp(indelem(idx(k)))+1:cp(indelem(idx(k))+1)];
        uDh = plob(:,1:p(indelem(idx(k)))+1)*x(index,:);
        xvals = ( s(k+1)+s(k) )/2 + pts*( s(k+1)-s(k) )/2;
        plot(xvals,uDh(:,1))   
        hold on
        if dim ==2
          plot(xvals,uDh(:,2),'r')
        end
        if isa(uD,'function_handle')
            %*** compute all evaluation points for u
            a = coordinates(elements(idx(k),1),:);
            b = coordinates(elements(idx(k),2),:);
            sx = (1-evalpts)'*a + evalpts'*b;
            u_sx = [u_sx; uD(sx,options)];
            s_sx = [s_sx;s(k)+evalpts'*h(indelem(idx(k)))];  
        end    
    end
    %*** optionally, we plot the exact solution
    if isa(uD,'function_handle')
       plot(s_sx,u_sx(:,1),'c--') 
       hold on;
       if dim ==2
            plot(s_sx,u_sx(:,2),'m--')
            hold on;
       end
    end
    %*** delete elements
    elements(idx,:)=[];
    indelem(idx)=[];
end 
%*** plot legend
hold off
eval([legstr,');'])    
set(leg,'location' , 'EastOutside')

