% plotArclengthP   plot a function in S \subset H^{-1/2}(\Gamma) defined by 
%                  its coefficients with respect to the Legendre polynomials
%                  over the arclenth of \Gamma. 
%
%   plotArclengthP(coordinates,elements,p,x [,phi] [,options] [,fig])
%
%   draws the solution given by the coefficient vector x for a boundary 
%   \Gamma which is specified by coordinates and elements. p (either a vector 
%   of length size(elements,1) or an integer >=0) species the polynomial 
%   degrees for the ansatz space S. 
%   phi is a function handle for the exact function phi, options is a
%   struct containing the Lame parameters and fig is the number of the
%   figure in which the plot should be created
%
%  Examples: 
%    elements    = [1,2; 2,3; 3,4];
%    coordinates = [0,0; 1,0; 1,1; 0,1];
%    x = [0,0.5,0.5,1,0,0].';
%    p = [0,1,2];
%    addpath('../laplace/mat') 
%    addpath('./c')
%    plotArclengthP(coordinates,elements,p,x, @(x,n,options) x(:,2));
%
% (c) epsBEM team 2012

function plotArclengthP(coordinates,elements,p,x,varargin)
if nargin < 4
  error('At least four input arguments required.')
else
  options = [];
  fig = 0;
  phi = false;
  for k = 1:nargin - 4
    if isstruct(varargin{k})
      options = varargin{k};
    elseif isa(varargin{k}, 'function_handle')
      phi = varargin{k};
    elseif isscalar(varargin{k}) && isnumeric(varargin{k})
      fig = varargin{k};
    end
  end
end
if fig > 0
    figure(fig);
 elseif fig == 0
    cla(gca);
end

nC = length(unique(elements)); % number of coordinates
nE = size(elements,1);         % number of elements
dim = size(x,2);               % problem dimension
p = p*zeros(nE,1);

%*** set parameters for the legend
legstr = 'leg = legend(';
if dim==1
  legstr = [legstr,'''\phi_h'''];
  plot(0,0); hold on
else
  legstr = [legstr,'''\phi_{h,1}'',''\phi_{h,2}'''];
  plot(0,0,'b', 0,0,'r'); hold on
end
if isa(phi,'function_handle')
  if dim ==1 
    plot(0,0,'--c')
    legstr = [legstr,',''\phi'''];
  else
    legstr = [legstr,',''\phi_{1}'',''\phi_{2}'''];
    plot(0,0,'--c',0,0,'--m')
  end  
end

%*** compute local mesh-size
h = sqrt(sum((coordinates(elements(:,1),:)-...
              coordinates(elements(:,2),:)).^2,2));
cp = cumsum([0;p(:)+1]);
s=0;
space=0;
%*** evaluate Legendre polynomials
pts = linspace(-0.95,0.95,5*(max(p)+1));
pleg = bLegendre(pts,max(p));

if isa(phi,'function_handle')
    %*** evaluation points for phi per element w.r.t. reference element [0,1]
    evalpts = linspace(0.1,0.9,200); 
end

indelem = 1:size(elements,1);
while ~isempty(elements)
    %*** find connected piece of the boundary Gamma 
    idx = findCycle(elements);
    s   = [0;cumsum(h(indelem(idx)))]+space+s(end);
    
    %*** plot discrete solution over arclength
    for k=1:length(idx)
        phih = pleg(:,1:p(indelem(idx(k)))+1)*...
                  x(cp(indelem(idx(k)))+1:cp(indelem(idx(k))+1),:);
        xvals = (s(k+1)+s(k))/2 + pts*(s(k+1)-s(k))/2;
        plot(xvals,phih(:,1))  
        hold on
        if dim ==2
            plot(xvals,phih(:,2),'r')
        end
        %*** optionally, we plot the exact solution
        if isa(phi,'function_handle')
            %*** compute all evaluation points for phi
            a = coordinates(elements(idx(k),1),:);
            b = coordinates(elements(idx(k),2),:);
            n = b-a;
            n = [n(2),-n(1)]./norm(n);
    
            sx = (1-evalpts)'*a + evalpts'*b;
           
            phi_sx = phi(sx,n,options);
            s_sx = s(k)+evalpts'*h(indelem(idx(k)));
            plot(s_sx,phi_sx(:,1),'c--')      
            if dim ==2
                plot(s_sx,phi_sx(:,2),'m--')
            end  
        end
    end
    space = ceil(0.05*sum(h));
    %*** delete elements
    elements(idx,:)=[];
    indelem(idx)=[];
end
%*** Plot legend
hold off
eval([legstr,');'])
set(leg,'location', 'EastOutside')