% potV   compute single layer potential in 2D.
%    Computes the single layer potential for the Laplace problem in 2D 
%    for one element.
%
%    V = potV(vertices,x,p[,ndigits])
%
%    Inputs: 
%       vertices - vertices of the element; 2-by-2 matrix s.t. each row
%                    corresponds to one vertex
%       x        - points at which the potential is evaluated; n-by-2 
%                    matrix s.t. each row corresponds to one point
%       p        - polynomial degree up to which the potential is
%                    calculated
%       ndigits  - if ndigits is specified the mpc routines with ndigits
%                    digits are used
%    Outputs: 
%       V       - n-by-(p+1) Matrix containing the values of the single 
%                    layer potential evaluated at the points x up to
%                    polynomial degree p
%                    V_i,j = VP_j(x_i)
%
%    Example:
%    addpath([pwd,filesep,'..',filesep,'..',filesep,'general',filesep,'c']);
%    vertices = [0,0; 1,0];
%    x = [1,1; 2,1; 1,2; 2,2];
%    potV(vertices, x, 10);
%
% (c) epsBEM team 2012

function V = potV(vertices,x,p, varargin)

m = (vertices(1,:)+vertices(2,:))/2;
u = (vertices(2,:)-vertices(1,:))/2;
utu = u*u';

w = [m(1)-x(:,1),m(2)-x(:,2)];
z_re = -(u(:,1).*w(:,1)+u(:,2).*w(:,2))/utu;
z_im = -(u(:,1).*w(:,2)-u(:,2).*w(:,1))/utu;

if(nargin==3)
    V = real(qtm1(complex(z_re,z_im),p,-sqrt(utu)/(2*pi)).');
else
    if isstruct(varargin{1}) && isfield(varargin{1},'ndigits')   
      V = real(qtm1_mpc(complex(z_re,z_im),p,-sqrt(utu)/(2*pi), varargin{1}.ndigits).');
    else
      V = real(qtm1(complex(z_re,z_im),p,-sqrt(utu)/(2*pi)).');
    end
end

V(:,1) = V(:,1) - sqrt(utu)/(2*pi)*log(utu);