% qtm1   Computes the integral
%
%            / 1
%            |    
%   fac(z) * |   P_j(t) * log(z-t) dt 
%            |
%           / -1
%
%   for complex z and j = 0,1,2,....,n
%
%   V = qtm1(z,n,fac) produces an (n+1)-by-numel(z) matrix that contains 
%   the weighted integral above.
%
%   Example:
%     z   = complex(1:2, 2:3);
%     fac = 1/(2*pi);
%     n   = 3; 
%     V   = qtm1(z,n,fac);
%
%     V   =  
%
%     2.6264e-01 + 3.6032e-01i   4.0986e-01 + 3.1653e-01i
%    -1.9473e-02 + 4.2008e-02i  -1.5883e-02 + 2.4558e-02i
%     2.6147e-03 + 3.0624e-03i   6.6360e-04 + 1.4678e-03i
%     4.8052e-04 - 1.3713e-04i   1.2535e-04 + 1.8543e-05i
%
% (c) epsBEM team 2012

function V = qtm1(z,n,fac)

z = z(:);
V=zeros(n+1,length(z));
if(numel(fac)==1)
  fac = fac*ones(size(z,1),1);
end

for j=1:length(z)
    V(:,j) = fac(j)*qtm1_int(z(j),n).';
end

function V = qtm1_int(z,k) 

% computes the integral 
%
%   / 1
%   |    
%   |   P_j(t) * log(z-t) dt 
%   |
%   / -1
%
% for complex z and j = 0,1,2,....,k  

u = real(z);
v = imag(z);

V = complex(zeros(1,k+1));
if ( v^2~=0 )
  tmp = log(z+1)-log(z-1);
  V(1) = log(z+1)+ log(z-1) + z*tmp - 2;
  if k>0
    V(2) = (z^2-1)/2*tmp-z;
  end
elseif u^2~=1
  tmp = log(abs(u+1)/abs(u-1));
  V(1) = log(abs(u+1)*abs(u-1)) + u*tmp - 2;
  if k>0
    V(2) = (u^2-1)*tmp/2-u;
  end
else
  V(1) = 2*log(2) - 2;
  if k>0
    V(2) = -u;  
  end
end

if k>1
  b = min(1,4.5/(k+1)^1.17); 
  a2 = 1+b^2;
  if ( u^2/a2+(v/b)^2 < 1 ) 
    % complex case
    if v ~= 0
      V(3) = 2/3 + z * V(2);
      for j=2:k-1
        V(j+2)=((2*j+1)*z*V(j+1)-(j-1)*V(j))/(j+2);
      end
    else
    % real case
      V(3) = 2/3 + u * V(2);
      for j=2:k-1
        V(j+2)=((2*j+1)*u*V(j+1)-(j-1)*V(j))/(j+2);
      end
    end
  else
    tol = 1e-14;  % tol check for neccesary value
    z2 = abs(u)+1i*abs(v);
    nu = k + ceil(log(1/tol)/(2*log(abs(z2+sqrt(z2^2-1)))));
    r = 1.0*ones(nu,1);
    for n = nu:-1:2   
      bn = ((n-1)*(n+1)) / ((2*n+1)*(2*n-1));
      r(n-1)= bn /(z-r(n));
    end 
    for n = 2:k 
      V(n+1) = ((2*n-1)/(n+1))*(r(n-1)*V(n));
    end
  end
end

