% r0  computes the integral 
%
%            / 1
%            |    
%   fac(z) * |   N_k(t)/(z-t) dt 
%            |
%           / -1
%
%   for complex z and k = 1,2,...., max(n+1,2)
%
%   V = r0(z,n,fac) produces an (n+1)-by-numel(z) matrix that contains 
%   the weighted integral above.
%
%   Example:
%     z   = complex(1:2, 2:3);
%     fac = 1/(2*pi);
%     n   = 3; 
%     V   = r0(z,n,fac);
%
%     V =
%
%     3.4155e-02 - 5.5159e-02i   2.5080e-02 - 3.3266e-02i
%     2.1004e-02 - 6.9841e-02i   2.1695e-02 - 4.0526e-02i
%    -1.9473e-02 + 4.2008e-02i  -1.5883e-02 + 2.4558e-02i
%     2.6147e-03 + 3.0624e-03i   6.6360e-04 + 1.4678e-03i
%
% (c) epsBEM team 2012

function V = r0(z,n,fac)

z = z(:);
V=zeros(max(n+1,2),length(z));
if(numel(fac)==1)
  fac = fac*ones(size(z,1),1);
end

for j=1:length(z)
    V(:,j) = fac(j)*r0_int(z(j),n).';
end


function val = r0_int(z,k)
% computes the integral 
%
%   / 1
%   |    
%   |   N_k(t)/(z-t) dt 
%   |
%   / -1
%
% for complex z = u+i*v and k = 0,1,2,....,k  
%
u = real(z);
v = imag(z);
val = zeros(1,max(k+1,2));
if ( v^2~=0 )
  tmp_log = log((z+1)/(z-1));
  val(1) = (1-z)/2*tmp_log+1;
  val(2) = (1+z)/2*tmp_log-1;
  if k>1
    val(3) = (z^2-1)/2*tmp_log-z;
  end
  if k>2
    val(4) = z*val(3)+2/3;
  end
elseif u^2~=1 
  tmp_log = log(abs((u+1)/(u-1)));
  val(1) = (1-u)/2*tmp_log+1;
  val(2) = (1+u)/2*tmp_log-1;
  if k>1
    val(3) = (u^2-1)/2*tmp_log-u;
  end
  if k>2
    val(4) = u*val(3)+2/3;
  end
elseif u==1 
  val(1) = 1;
  val(2) = Inf;
  if k>1
    val(3) = -u;
  end
  if k>2
    val(4) = u*val(3)+2/3;
  end
else % u = -1
  val(1) = -Inf;
  val(2) = -1;
  if k>1
    val(3) = -u;
  end
  if k>2
    val(4) = u*val(3)+2/3;
  end

end

if k>3
  b = min(1,4.5/(k+1)^1.17);  
  a = sqrt(1+b^2);
  if ( (u/a)^2+(v/b)^2 < 1 ) % forward recurrence  
% complex case
    if v^2~=0
      for j=2:k-2
        val(j+3)=((2*j+1)*z*val(j+2)-(j-1)*val(j+1))/(j+2);
      end
    else
% real case
      for j=2:k-2
        val(j+3)=((2*j+1)*u*val(j+2)-(j-1)*val(j+1))/(j+2);
      end
    end
  else
    tol = 1e-14;
    z2 = abs(u)+1i*abs(v);
    nu = k + ceil(log(1/tol)/(2*log(abs(z2+sqrt(z2^2-1)))));
    r = ones(nu,1);
    for n = nu:-1:3   
      bn = ((n-1)*(n+1))/ ((2*n+1)*(2*n-1));
      r(n-1)= bn /(z-r(n));
    end 
    for n = 3:k-1  
      val(n+2) = (2*n-1)/(n+1)*r(n-1)*val(n+1);
    end
  end
end