% refineUniform  uniform mesh refinement
%
%    [coordinates,varargout] = refineUniform(coordinates,varargin)
%    performs uniform refinement of the boundary mesh. 
%    The input parameters are the vertices of the boundary stored in 
%    coordinates and the elements and polynomial degrees for the 
%    different boundary parts: 
%    varargin = {elements1,p1, elements2,p2, ...}
%    The function returns the vertices of the refined mesh in 
%    coordinates and the refined elements and the polynomial degrees for
%    the different bopundary parts:
%    varargout = {elements1,p1,[f2s1,] elements2,p2,[f2s2,] ...}
%
% Example 
%    [c,e,p,e2,p2] = refineUniform([0,0; 1,0; 2,0], [1,2], 2, [2,3], 3)
%
%     c =
%              0         0
%         1.0000         0
%         2.0000         0
%         0.5000         0
%         1.5000         0
%     e =
%          1     4
%          4     2
%     p =
%          2
%          2
%     e2 =
%          2     5
%          5     3
%     p2 =
%          3
%          3
%
% (c) epsBEM team 2012


function [coordinates,varargout] = refineUniform(coordinates,varargin)

if mod(nargin,2) == 0
  error('Input expected like (coordinates1, elements1, p1, elements2,p2,...).')
end

nC = size(coordinates,1);
nBdry = (nargin-1)/2;

%*** obtain set of all elements of the boundary partition
elements = cat(1,varargin{1:2:2*nBdry});

%*** compute and add new nodes
coordinates = [ coordinates; ...
               (coordinates(elements(:,1),:) ...
              + coordinates(elements(:,2),:))/2];

%*** indices of a boundary part w.r.t. entire field elements
nE_boundary = zeros(1,nBdry);
for iter = 1:nBdry
  nE_boundary(iter) = size(varargin{2*iter-1},1);
end

ptr_boundary = cumsum([0,nE_boundary]); 
 
%*** check, if user asks for father2son fields in output
if nargout == (2*nBdry+1)
    f2s = 0;
else
    f2s = 1;
end

%*** uniform refinement of mesh iterates over each boundary part
for iter = 1:nBdry
    %*** generate new elements
    idx = (ptr_boundary(iter)+1):ptr_boundary(iter+1);
    varargout{(2+f2s)*(iter-1)+1} = [ varargin{2*iter-1}(:,1), nC + idx' ;
                                     nC + idx', varargin{2*iter-1}(:,2) ];
                          
    %*** generate new polynomial degrees
    p0 = varargin{2*iter}*ones(nE_boundary(iter),1);
    varargout{(2+f2s)*(iter-1)+2}   = [p0;p0];
    
    %*** compute father2son only if desired
    if f2s == 1
        varargout{(2+f2s)*(iter-1)+3} ...
            = [(1:nE_boundary(iter))', ...
               (1:nE_boundary(iter))' + nE_boundary(iter)];
    end
    
end
